//+------------------------------------------------------------------+
//|                                                   Rabinovich.mq4 |
//|                                            Copyright  2008, OOO |
//|                                                   777768@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Rabinovich"
#property link      "777768@mail.ru"

// Direction defines
#define NONE   0
#define UP     1
#define DN     -1

// Magic
extern int        Magic             =  12345;

extern double     Lots              =  0;
extern int        RiskPercentage    =  0;
extern int        Slippage          =  1;

extern int        MaxOrders         =  1;

extern int        TakeProfit        =  15;

extern int        TrailingPeriod    =  10;
extern int        UseTrailing       =  0;

extern int        LifeTimeMinutes   =  30;

double LotsToBid;
string symbol;

double GetLotsToBid(int RiskPercentage)
{
   double margin = MarketInfo(Symbol(), MODE_MARGINREQUIRED);
   double minLot = MarketInfo(Symbol(), MODE_MINLOT);
   double maxLot = MarketInfo(Symbol(), MODE_MAXLOT);
   double step =   MarketInfo(Symbol(), MODE_LOTSTEP);
   double account = AccountFreeMargin();
   
   double percentage = account*RiskPercentage/100;
   
   double lots = MathRound(percentage/margin/step)*step;
   
   if(lots < minLot)
   {
      lots = minLot;
   }
   
   if(lots > maxLot)
   {
      lots = maxLot;
   }

   return (lots);
}

int GetOrdersCount(int MagicNumber, int Type)
{
   int count = 0;
   
   for(int i = 0; i < OrdersTotal(); i++)
   {
      // already closed
      if(OrderSelect(i, SELECT_BY_POS) == false) continue;
      // not current symbol
      if(OrderSymbol() != Symbol()) continue;
      // order was opened in another way
      if(OrderMagicNumber() != MagicNumber) continue;
      
      if(OrderType() == Type)
      {
         count++;
      }
   }
   
   return (count);
}

void CheckForExpired()
{
   for(int i = OrdersTotal() - 1; i >= 0; i--)
   {
      // already closed
      if(OrderSelect(i, SELECT_BY_POS) == false) continue;
      // not current symbol
      if(OrderSymbol() != Symbol()) continue;
      // order was opened in another way
      if(OrderMagicNumber() != Magic) continue;
      
      if (TimeCurrent() - OrderOpenTime() > LifeTimeMinutes*60)
      {
         if(OrderType() == OP_BUYSTOP || OrderType() == OP_SELLSTOP)
         {
            OrderDelete(OrderTicket());
         }
      }
   }
}

int GetStopLevel()
{
   return (MathMax(MathRound(MarketInfo(symbol, MODE_STOPLEVEL)), 1));
}

void CheckLosses(int MN)
{
   double spread = Ask - Bid;
   double sellLoss = High[iHighest(symbol, 0, MODE_HIGH, TrailingPeriod + 1)] + spread;
   double buyLoss = Low[iLowest(symbol, 0, MODE_LOW, TrailingPeriod + 1)];
   
   int stopLevel = GetStopLevel() + Slippage;
   
   if (NormalizeDouble((sellLoss - Bid)/Point, 0) < stopLevel)
   {
      sellLoss = Bid + stopLevel*Point;
   }

   if (NormalizeDouble((Ask - buyLoss)/Point, 0) < stopLevel)
   {
      buyLoss = Ask - stopLevel*Point;
   }

   for(int i = 0; i < OrdersTotal(); i++)
   {
      // already closed
      if(OrderSelect(i, SELECT_BY_POS) == false) continue;
      // not current symbol
      if(OrderSymbol() != symbol) continue;
      // order was opened in another way
      if(OrderMagicNumber() != MN) continue;
   
      double currentLoss = OrderStopLoss();
      
      if(OrderType() == OP_SELL)
      {
            if (currentLoss > sellLoss || currentLoss == 0)
            {
               OrderModify(OrderTicket(), OrderOpenPrice(), sellLoss, OrderTakeProfit(), 0);
               RefreshRates();
            }
      }

      if(OrderType() == OP_BUY)
      {
            if (currentLoss < buyLoss || currentLoss == 0)
            {
               OrderModify(OrderTicket(), OrderOpenPrice(), buyLoss, OrderTakeProfit(), 0);
               RefreshRates();
            }
      }
   }
}

void OpenBuyStop(int MN, double Price, int Target, int Loss)
{
   Price = Price + Ask - Bid;

   double TP = 0;
   if (Target > 0)
   {
      TP = Price + Target*Point;
   }

   double SL = 0;
   if (Loss > 0)
   {
      SL = Price - Loss*Point - Ask + Bid;
   }
   
   if (Lots == 0) LotsToBid = GetLotsToBid(RiskPercentage);
   
   OrderSend(Symbol(), OP_BUYSTOP, LotsToBid, Price, Slippage, SL, TP, NULL, MN, 0, Blue);
}

void OpenSellStop(int MN, double Price, int Target, int Loss)
{
   double TP = 0;
   if (Target > 0)
   {
      TP = Price - Target*Point;
   }

   double SL = 0;
   if (Loss > 0)
   {
      SL = Price + Loss*Point;
   }
   
   if (Lots == 0) LotsToBid = GetLotsToBid(RiskPercentage);
   
   OrderSend(Symbol(), OP_SELLSTOP, LotsToBid, Price, Slippage, SL, TP, NULL, MN, 0, Red);
}

void Check()
{
   double RFTL1 = iCustom(symbol, 0, "RFTL", 0, 1);
   double RFTL2 = iCustom(symbol, 0, "RFTL", 0, 2);
   
   if (RFTL1 == 0 || RFTL1 == EMPTY_VALUE) return;
   if (RFTL2 == 0 || RFTL2 == EMPTY_VALUE) return;
   
   //   
   if (Close[2] >= RFTL2 && Close[1] < RFTL1)
   {
      //CloseBuys(Magic, Slippage);
      //  ,      
      
      //   ,    
      int count = GetOrdersCount(Magic, OP_BUY) + GetOrdersCount(Magic, OP_SELL);
      
      if (count < MaxOrders || MaxOrders == 0) 
      {
         Print("Sell");

         OpenSellStop(Magic, Low[1] - Point, TakeProfit, MathRound((High[1] - Low[1])/Point) + 2);
      }
   }

   //   
   if (Close[2] <= RFTL2 && Close[1] > RFTL1)
   {

      //CloseSells(Magic, Slippage);
      //  ,      
   
      //   ,    
      count = GetOrdersCount(Magic, OP_BUY) + GetOrdersCount(Magic, OP_SELL);
      
      if (count < MaxOrders || MaxOrders == 0)
      {
         Print("Buy");

         OpenBuyStop(Magic, High[1] + Point, TakeProfit, MathRound((High[1] - Low[1])/Point) + 2);
      }
   }
}

int init()
{
   LotsToBid = Lots;
   symbol = Symbol();
}

datetime LastTime = 0;

int start()
{
   // Trading only when tick is opening
   if (LastTime < Time[0])
   {
      LastTime = Time[0];
      
      CheckForExpired();
      
      // Check for open new orders and close current ones
      if (UseTrailing == 1) CheckLosses(Magic);
      
      Check();
   }

   return(0);
}